
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity simplecpu_v1a is
port(
  CLK    : in  STD_LOGIC;
  CLR    : in  STD_LOGIC;
  ROM_EN : out STD_LOGIC;   
  RAM_EN : out STD_LOGIC;
  RAM_WR : out STD_LOGIC;
  ADDR   : out STD_LOGIC_VECTOR(7 downto 0); 
  DIN    : in  STD_LOGIC_VECTOR(15 downto 0); 
  DOUT   : out STD_LOGIC_VECTOR(15 downto 0) ); 
end simplecpu_v1a;

architecture simplecpu_v1a_arch of simplecpu_v1a is

  component CONTROL_LOGIC 
  port (
    CLK      : in  STD_LOGIC;                 
    CLR      : in  STD_LOGIC;                
    D        : in  STD_LOGIC_VECTOR(3 downto 0);    
    Z        : in  STD_LOGIC; 
    ROM_EN   : OUT STD_LOGIC;  
    RAM_EN   : OUT STD_LOGIC;  
    RAM_WR   : OUT STD_LOGIC;   
    ADDR_SEL : OUT STD_LOGIC;  
    DATA_SEL : OUT STD_LOGIC; 
    ALU_CTL0 : OUT STD_LOGIC; 
    ALU_CTL1 : OUT STD_LOGIC;  
    ALU_CTL2 : OUT STD_LOGIC;  
    ACC_EN   : OUT STD_LOGIC;  
    IR_EN    : OUT STD_LOGIC; 
    PC_LD    : OUT STD_LOGIC;  
    PC_EN    : OUT STD_LOGIC );  
  end component;
  
  component ALU 
  port (
    A, B : in  STD_LOGIC_VECTOR(7 downto 0); 
    CTL  : in  STD_LOGIC_VECTOR(2 downto 0);                  
    Y    : out STD_LOGIC_VECTOR(7 downto 0) );
  end component;
  
  component COUNTER_8 
  port (
    CLK : in  STD_LOGIC;                 
    CLR : in  STD_LOGIC;                
    LD  : in  STD_LOGIC;  
    CE  : in  STD_LOGIC;  	 
    D   : in  STD_LOGIC_VECTOR(7 downto 0); 
    Q   : out STD_LOGIC_VECTOR(7 downto 0) );
  end component;
  
  component REG_16 
  port (
    D   : in STD_LOGIC_VECTOR(15 downto 0);         
    Q   : out STD_LOGIC_VECTOR(15 downto 0);
    CLK : in STD_LOGIC;  
    CLR : in STD_LOGIC;
    CE  : in STD_LOGIC );
  end component;
  
  component REG_8 
  port (
    D   : in STD_LOGIC_VECTOR(7 downto 0);         
    Q   : out STD_LOGIC_VECTOR(7 downto 0);
    CLK : in STD_LOGIC;  
    CLR : in STD_LOGIC;
    CE  : in STD_LOGIC );
  end component;
  
  component MUX_2_8 
  port (
    A, B : IN  STD_LOGIC_VECTOR(7 downto 0); 
    SEL  : IN  STD_LOGIC;                    
    Y    : OUT STD_LOGIC_VECTOR(7 downto 0));
  end component;
  
  component NOR_8 
  port( A : IN STD_LOGIC_VECTOR( 7 downto 0 );
      Z : OUT STD_LOGIC);
  end component;

  signal ir_int       : STD_LOGIC_VECTOR( 15 downto 0 );

  signal acc_int      : STD_LOGIC_VECTOR( 7 downto 0 );
  signal data_mux_int : STD_LOGIC_VECTOR( 7 downto 0 );  
  signal alu_int      : STD_LOGIC_VECTOR( 7 downto 0 );
  signal pc_int       : STD_LOGIC_VECTOR( 7 downto 0 );

  signal alu_ctl      : STD_LOGIC_VECTOR( 2 downto 0 );
  
  signal z_int    : STD_LOGIC;
  signal acc_en   : STD_LOGIC;
  signal ir_en    : STD_LOGIC;
  signal pc_en    : STD_LOGIC;
  signal pc_ld    : STD_LOGIC;
  signal addr_sel : STD_LOGIC;
  signal data_sel : STD_LOGIC; 
  

begin

  instruction_reg : REG_16 port map(
    D   => DIN,      
    Q   => ir_int,
    CLK => CLK, 
    CLR => CLR,
    CE  => ir_en );
	 
  ctrl_logic : CONTROL_LOGIC port map(
    CLK      => CLK,                    
    CLR      => CLR,                    
    D        => ir_int(15 downto 12),     
    Z        => z_int,
    ROM_EN   => ROM_EN,  
    RAM_EN   => RAM_EN,  
    RAM_WR   => RAM_WR,   
    ADDR_SEL => addr_sel,
    DATA_SEL => data_sel,
    ALU_CTL0 => alu_ctl(0),
    ALU_CTL1 => alu_ctl(1),
    ALU_CTL2 => alu_ctl(2),
    ACC_EN   => acc_en,
    IR_EN    => ir_en,
    PC_LD    => pc_ld,
    PC_EN    => pc_en );  

  program_counter : COUNTER_8 port map(
    CLK => CLK,              
    CLR => CLR,              
    LD  => pc_ld, 
    CE  => pc_en,  
    D   => ir_int(7 downto 0),
    Q   => pc_int );
	 
  addr_mux : MUX_2_8 port map(
    A   => pc_int,
	 B   => ir_int(7 downto 0),
    SEL => addr_sel,                    
    Y   => ADDR );
	 
  arith_logic_unit : ALU port map(
    A   => acc_int,
	 B   => data_mux_int,
    CTL => alu_ctl,                 
    Y   => alu_int );
 
  acc : REG_8 port map(
    D   => alu_int,        
    Q   => acc_int,
    CLK => CLK,  
    CLR => CLR,
    CE  => acc_en );
	 
  zero : NOR_8 port map( 
    A => acc_int,
    Z => z_int );
	 
  DOUT <= "00000000" & acc_int;
	 
  data_mux : MUX_2_8 port map(
    A   => ir_int(7 downto 0),
	 B   => DIN(7 downto 0),
    SEL => data_sel,                    
    Y   => data_mux_int );

end simplecpu_v1a_arch;

